/*
CPS606 Assgn # 2
Rational.java  - A Java Application for performing arithmetic with fractions

Author : Xiaohong Qiu (cu98xqi)
*/

public class Rational {

 /* Private integer variables of the class. num denotes the numerator and
    den denotes the denominator
 */
 private int num,den;

 /* Constructor that takes in a numerator and a denominator and reduces the
    values to the lowest values
    param n integer for numerator
    param d integer for denominator 
 */
 public Rational(int n, int d) {
     num = n;
     den = d;
     this.reduce();
 } //end of Constructor 
 
 /* Constructor that does not take any arguments and initializes the object
    to a default value of 0 
 */
 public Rational(){
     num = 0;
     den = 1;
 } // end of Constrcutor
 
 /* ----------------------------------------------------
     Get and Set methods for the variables of the class which could be
     called  to get and set the values of the instance variables.
 */

 // Returns the numerator instance variable

 public int getNum() {
     return num;
 }

 // Returns the denominator instance variable
 public int getDen() {
     return den;
 }

 // Sets the numerator instance variable
 // param i integer 
 public void setNum(int i) {
     num = i;
 }

 // Sets the denominator instance variable
 // param i integer 
 public void setDen(int i) {
     den = i;
 }

 /* End of Get and Set Methods
    -------------------------------------------------- */

 /* private reduce function accessed by other methods in the class to
    reduce the values of the numberator and the denominator to the lowest
    values
 */
 private void reduce() {
     int a,b,c; //local variables 
     a = this.getNum(); //get the numberator and denominator
     b = this.getDen();
     if(a > 0 && b > 0) { c = gcd(a,b); } // get the GCD of the numbers
     else {c = 0;}

     if(c > 0) {       // if GCD is greater than zero then divide the
         a = a /c;     // numbers by the GCD
         b = b /c;
     }
     this.setNum(a);   // set the variable values back to the reduced
                       //values
     this.setDen(b);
 }

 /* private function gcd to find the greatest common divisor of two numbers
    taken in as arguments and the greatest common divisor is returned back
    Took directly from the assign problem
 */
 private int gcd(int u, int v) {
     int t;
     while(u > 0){
         if(u < v) { t = u; u = v; v = t; }
         u = u - v;
     }   
     return v;
 }
 

 /* finds the sum of the two rational numbers r1 and r2 and 
    returns a rational number
    param r1 rational number
    param r2 rational number
    returns rational number
 */
 public static Rational sum(Rational r1, Rational r2) {
     int r1n, r1d, r2n, r2d, nm, dn;

     r1n = r1.getNum();
     r1d = r1.getDen();
     r2n = r2.getNum();
     r2d = r2.getDen();

     nm = (r1n * r2d) + (r2n * r1d);
     dn = r1d * r2d;
     
     return new Rational(nm,dn);
 }

 /* finds the product of the two rational numbers r1 and r2 and 
    returns a rational number
    param r1 rational number
    param r2 rational number
    returns rational number
 */
 public static Rational multiply(Rational r1, Rational r2) {
     int r1n, r1d, r2n, r2d, nm, dn;

     r1n = r1.getNum();
     r1d = r1.getDen();
     r2n = r2.getNum();
     r2d = r2.getDen();

     nm = r1n * r2n;
     dn = r1d * r2d;
     
     return new Rational(nm,dn);
 }

 /* subtracts two rational numbers r1 and r2 and returns the result as a
    rational number
    param r1 rational number
    param r2 rational number
    returns rational number

 */
 public static Rational subtract(Rational r1, Rational r2) {
     int r1n, r1d, r2n, r2d, nm, dn;

     r1n = r1.getNum();
     r1d = r1.getDen();
     r2n = r2.getNum();
     r2d = r2.getDen();

     nm = (r1n * r2d) - (r2n * r1d);
     dn = r1d * r2d;
     
     return new Rational(nm,dn);
 }

 /* compares two rational numbers r1 and r2
    returns 1 if r1 < r2
            0 if r1 = r2
           -1 if r1 > r2 
    param r1 rational number
    param r2 rational number
    returns integer
 */
 public static int compare(Rational r1, Rational r2) {
     int r1n, r1d, r2n, r2d;

     r1n = r1.getNum();
     r1d = r1.getDen();
     r2n = r2.getNum();
     r2d = r2.getDen();

     if((r1n * r2d) < (r2n * r1d)) { return 1; }
     else if ((r1n * r2d) > (r2n * r1d)) { return -1; }
     else return 0;
 }

 /* converts a rational number to a string
    uses the this object instance
 */
 public String toString() {
     String tmp = "";
     tmp = Integer.toString(this.getNum())+"/"+Integer.toString(this.getDen());
     return tmp;
 }

 /* execution of the program starts here 
    Used as a test stub when this file is executed 
 */
 public static void main(String[] args) {
     Rational d ;
     Rational d1 = new Rational(10,100);
     Rational d2 = new Rational(3,30);
     Rational d3 = new Rational(0,12);
     Rational d4 = new Rational(15,5);

     d = Rational.sum(d1,d2);
     System.out.println("Sum = "+ d.toString());
     d = Rational.subtract(d1,d2);
     System.out.println("Subtract = " +d.toString());
     d = Rational.multiply(d1,d2);
     System.out.println("Multiply = " +d.toString());
     System.out.println("Compare " + Rational.compare(d1,d2));

 }

}


