/**
 * Connection.java
 *
 * This class is the client or applet side of the client-server system.
 * It is used for opening sockets to the server and sending and receiving
 * information back and forth to the server. All the methods are made
 * public so that  the user interface classes can call them after
 * instantiating a Connection object. The port number and hostname are
 * obtained from the  static attributes in the ReservationApplet class which are
 * in turn obtained from the applet's html file.
 *
 * @author xjqiu@npac.syr.edu
 *
 */

package client;

import java.io.*;  // for io stream objects
import java.net.*; // for sockets
import gui.*;      // the gui classes for ReservationApplet reference

public class Connection { // class definition

 private Socket client;  // declare a socket
 private DataInputStream in; // input and output streams
 private DataOutputStream out;
 private boolean connection = false;

/**
 * Constructor called by the user interface classes. The class uses the 
 * static members in the applet class to get the port number and hostname
 * to connect to the server.
 */

 public Connection() {  
     this(ReservationApplet.host,ReservationApplet.port); // uses the other constructor
 }

/**
 * Constrcutor which takes in a port number and a hostname to 
 * open sockets to the server.
 */

 public Connection(String host, int port) { //constructor
     try {

         client = new Socket(host,port); //open socket to the server
         connection = true;

         // get the data output stream from the socket
         out = new DataOutputStream(client.getOutputStream());

         // get the data input stream from the socket
         in = new DataInputStream(client.getInputStream());

     }catch(IOException e) { // catch thrown exception and print it
         msg("Error while connecting to server !!!");
         msg(e.getMessage());
     }
 }
/**
 * public method to check if the connection is there or not.
 */
 
 public boolean isConnected() {
     return connection;
 }

/**
 * Public method called by the gui classes for sending information to the
 * server. It uses the writeUTF method to write data to the output stream.
 * Returns false if there is any error while sending.
 */

 public boolean sendInfo(String str) {
     try{
         out.writeUTF(str); // write data to stream
     }catch (Exception e) {
         msg("Error !!! While sending information to server...");
         msg(e.getMessage());
         return false; //return false if an exception occurs
     }
     return true; //true otherwise
 }

/**
 * Public method called by the gui classes for reading information from the
 * server. It uses the readUTF method to read data from the input stream.
 * Returns  the string that was read from the stream.
 */

 public String readInfo() {
     String str = "";
     try{
         str = in.readUTF(); //read data from the stream
     }catch(Exception e) {
         msg(e.getMessage()); // returns an error message
         return "Error while reading information from server...";
     }
     return str; // return the string to the caller
  }

/**
 * Method called by the classes using the Connection object to
 * close the socket connection to the server. Returns true if it was
 * a success or false otherwise.
 */

 public boolean closeConnection() {
     try{
         client.close(); //close the socket connection
         connection = false;
     }catch(Exception e) {
         msg("Error !!! while closing conection ..");
         msg(e.getMessage());
         return false;
     }
     return true; // true if successful
 }
 
/**
 * This method is used for printing messages on the console window if using
 * an appletviewer or Java Console if using a netscape window...
 */
 private void msg(String str) {
     System.out.println(str);
 }

} // end of class
