/**
 * NewForm.java
 *
 * This class represents the form for adding a new record to the database. This uses a
 * GridBagLayout and places the various textfield and list components in the panel 
 * appropriately. It uses the Connection class to connect to the server and send the
 * data to the server. This class is called when the New button in the left panel is
 * clicked. It implements the ActionListener interface so that the Submit and Cancel 
 * button clicks can be trapped.
 * 
 * @author xjqiu@npac.syr.edu
 */
package gui; //define the package

import java.awt.*;
import java.awt.event.*;
import client.*; // for the Connection class

public class NewForm extends Panel  // class definition
        implements ActionListener {

 GridBagLayout gb = new GridBagLayout();  // use a GridBagLayout
 GridBagConstraints gbc = new GridBagConstraints();  // constraints for the layout

 private Color fg = new Color(139,0,139); //foreground color
 private CityList from,to; //components for citylist
 private Choice time; // time selection list
 private TextField day,name,ph; // text boxes for day, name, ph
 private Button submit,cancel; // buttons for submit and cancel
 private Label l1,l2, l3, l4, l5, l6; // labels for the textboxes and buttons
 
 private StatusWin swin; // Status Window for client server interactions

 private Connection con; // Connection class

/**
 * Constructor for the class. Sets up the various components in the panel
 * appropriately according to the constraints.
 */

 public NewForm() {

     setLayout(gb);  //sets the layout
     setBackground(Color.white); //sets background color

     gbc.fill = GridBagConstraints.NONE;
     gbc.anchor = GridBagConstraints.NORTHWEST; // sets anchor as NORTHWEST

     setTime(); // makes the time choice list
     
     l1 = makeLabel(" From  :"); //add from label and city list
     addComp(l1,4,0,1,1,new Insets(-215,-45,1,1));
     from = new CityList("SYR");
     from.setFont(new Font("Helvetica", Font.BOLD, 12));
     from.setBackground(fg);
     from.setForeground(Color.white);
     addComp(from,9,0,1,1,new Insets(-215,2,1,1));

     l2 = makeLabel(" To :"); //add to label and city list
     addComp(l2,40,0,1,1,new Insets(-215,1,1,1));
     to = new CityList("BOS");
     to.setFont(new Font("Helvetica", Font.BOLD, 12));
     to.setBackground(fg);
     to.setForeground(Color.white);
     addComp(to,60,0,1,1,new Insets(-215,1,1,1));

     l3 = makeLabel(" Time   : "); //add time label and list
     addComp(l3,4,2,1,1,new Insets(-165,-45,1,1));
     addComp(time,9,2,1,1,new Insets(-165,2,1,1));
 
     l4 = makeLabel(" Day    : "); //adds day, name and ph text boxes
     addComp(l4,4,5,1,1,new Insets(-135,-45,1,1));
     day = new TextField(10);
     addComp(day,9,5,1,1,new Insets(-135,2,1,1));

     l5 = makeLabel(" Name : ");
     addComp(l5,4,7,1,1,new Insets(-105,-45,1,1));
     name = new TextField(20);
     addComp(name,9,7,1,2,new Insets(-105,2,1,1));

     l6 = makeLabel(" PH    #  ");
     addComp(l6,4,9,1,1,new Insets(-75,-45,1,1));
     ph = new TextField(10);
     addComp(ph,9,9,1,1,new Insets(-75,2,1,1));

     submit = new Button("Submit"); //add submit and cancel and their action listeners
     submit.setBackground(fg);     // which is this class itself
     submit.setForeground(Color.white);
     submit.setFont(new Font("Helvetica", Font.BOLD, 14));
     submit.addActionListener(this);
     addComp(submit,4,11,1,2,new Insets(-25,-45,1,1));
     cancel = new Button("Cancel");
     cancel.setBackground(fg);
     cancel.setForeground(Color.white);
     cancel.setFont(new Font("Helvetica", Font.BOLD, 14));
     cancel.addActionListener(this);
     addComp(cancel,9,11,1,3,new Insets(-25,1,1,1));

     swin = new StatusWin(); // adds a status window compoenent to the panel
     addComp(swin,0,402,0,0,new Insets(70,-15,-450,3));

     // initialize the status window with a message
     swin.writeMsg("Status of the client  <--> server interactions ...");
 }

/**
 * paint method called by the browser for drawing a separator line
 */

 public void paint(Graphics g) {
     g.drawLine(0,300,500,300);
 }

 /**
 * Method that actually sets the parameters and adds the component
 * to the panel. Gets the coordinates, width and height and insets
 * as well as the component object as arguments.
 */

 private void addComp
            ( Component c, int x, int y, int ht, int wt, Insets insets ) {
     gbc.gridx = x;
     gbc.gridy = y;

     gbc.gridwidth = wt;
     gbc.gridheight = ht;

     gbc.insets = insets;

     gb.setConstraints(c,gbc);
     add(c);
 }

/**
 * This method constructs a time choice list with four times
 */

 private void setTime() {
     time = new Choice();
     time.setFont(new Font("Helvetica", Font.BOLD, 12));
     time.setBackground(fg);
     time.setForeground(Color.white);
     time.add("6.00 AM");
     time.add("7.00 AM");
     time.add("5.00 PM");
     time.add("6.00 PM");
 }

/**
 * Helper method in constructing labels. Sets the foreground and background colors
 * fonts etc.
 */

 private Label makeLabel(String str) {
     Label l = new Label(str);
     l.setForeground(fg);
     l.setFont(new Font("Helvetica", Font.BOLD, 14));
     return l;
 }

/**
 * The Action Listener implementation which is called when the submit and cancel
 * buttons are clicked.  It uses the doPost method to contact the server if the submit
 * button had been clicked.
 */

 public void actionPerformed(ActionEvent e) {
     Button b = (Button) e.getSource(); //get the button 
   
     String tmpstr = b.getLabel(); // get the label of the button

     if(tmpstr.equals("Submit")) { // check if it is submit and call doPost
         clearStatusWin();//clear the status window
         doPost();
     }else if(tmpstr.equals("Cancel")) { //check if it is cancel
         clearAll();
     }
     b = null; 
     tmpstr = null;
 }

/**
 * This method uses the Connection class to actually contact the server and post the
 * data to the server. It also checks for the data consistency  and updates the status
 * window accordingly.
 */

 private void doPost() {
     // check if from and to are the same
     if(from.getSelectedItem().equals(to.getSelectedItem())){
         swin.writeMsg("Error ! From and To selections are same !!!");       
         return;
     }
     // check if all fields have been entered
     if((day.getText().trim()).length() <= 0 || 
                 (name.getText().trim()).length()<= 0 ||
                  (ph.getText().trim()).length() <= 0 ) {
         swin.writeMsg("Error ! Textfields cannot be empty !!!");
         return;
     }
     //send updates to the status window
     // and try to connect to the server
     swin.writeMsg("Connecting to the server....");
     con = new Connection();
     if(!con.isConnected() ){
         swin.writeMsg("Error !!! while connecting to server....");
         return;
     }
     // send the first line as action=Post 
     // which the server is expecting
     swin.writeMsg("Sending information to server...");
     if(! con.sendInfo("action=Post")) {
         swin.writeMsg("Error !!! while sending information to server...");
         return;
     }
     // send the actual data after sending the action parameter
     con.sendInfo(getValues());
     // update the status window with progress
     swin.writeMsg("Send complete....");
     setDefault();
     //read the acknowledgement and update the status window
     swin.writeMsg("Reading response from server....");
     swin.writeMsg("         "+con.readInfo());
     // close the connection
     swin.writeMsg("Closing connection to server...");
     con.closeConnection();
 }

/**
 * This method gets the values from the various text boxes 
 * and list components.
 */

 private String getValues() {
     StringBuffer sb = new StringBuffer();
                               // construct a big string from the textfield and list
                               // field values 
     sb.append("from="+from.getSelectedItem()+"##");
     sb.append("to="+to.getSelectedItem()+"##");
     sb.append("time="+time.getSelectedItem()+"##");
     sb.append("day="+day.getText()+"##");
     sb.append("name="+name.getText()+"##");
     sb.append("ph="+ph.getText());
     return sb.toString(); // return the string
 }

/**
 * This method clears all the text boxes and sets default values to the 
 * list boxes.
 */

 public void clearAll() {
     setDefault();
     clearStatusWin();
 }

/**
 * This method clears all the text boxes and sets default values for the choice lists
 */

 private void setDefault(){
     from.select("SYR");
     to.select("BOS");    
     time.select(0);
     day.setText("");
     name.setText("");
     ph.setText("");
 }

/**
 * This method clears the status window
 */

 private void clearStatusWin() {
     swin.setText("Status of the client  <--> server interactions ...");
 }

} // end of class
