/**
 * RightPanel.java
 *
 * This class is the panel in the right which uses a card layout to 
 * display the appropriate card depending on the button clicked on the left frame.
 *
 * @author xjqiu@npac.syr.edu
 */
package gui; //define package

import java.awt.*;
import java.awt.event.*;

public class RightPanel extends Panel  // class definition
       {

 private CardLayout cd ; //declare a card layout
 private PlainPage pp; // declare the various cards 
 private ListForm lform;
 private NewForm nform;

/**
 * Constructor for the class, builds up the cards and shows a default card.
 */

 public RightPanel() {
     cd = new CardLayout();
     setLayout(cd); // set the layout

     pp    = new PlainPage(); //create instances for the cards
     lform = new ListForm();
     nform = new NewForm();

     add("Cancel",pp); // add cards to the layout
     add("List",lform);
     add("New",nform);

     cd.show(this,"Cancel"); //show the default card
 }

/**
 * This is a public method called from the MainPanel when a corresponding button
 * is clicked in the left panel to show to appropriate card. It clears up the forms
 * in the other cards while flipping the cards
 */

 public void showCard(String cname) {
     nform.clearAll(); //clear forms
     lform.clearAll();
     cd.show(this,cname.trim()); //show the card requested
 }
} // end of class
