/**
 * Server.java
 *
 * This is the main server class that starts up and listens to client
 * connections. It reads the port number where the server is running 
 * from the command line and starts the server on that port. If no port
 * number is given then the default port of 4321 is used. 
 * 
 * @author xjqiu@npac.syr.edu
 * 
 */
 
package server; // server package

import java.net.*; // for socket connections
import java.io.*; //IOException

public class Server { //class definition

 private ServerSocket server; // declare a ServerSocket
 private Socket connection;   // declare a Socket 

/**
 * Constructor for the class that takes in a port number and creates 
 * a ServerSocket object on that port and starts listening to client
 * connections.  When a connection is established, use the
 * ConnectionHandler class to  handle the request.
 */
 public Server(int port) {

     try {
         server = new ServerSocket(port); // try creating a server socket
         msg("Server started on port "+ port );
     }catch(IOException e) { //if an error occurs print that
         msg("Error starting server !!!");
         msg(e.getMessage());
     }

     try {

         while(true) { // start the infinite loop for listening to
                      // client connections and accept them

             connection = server.accept();
             msg("Accepted connection from "+
                       connection.getInetAddress().getHostName());
             
             new ConnectionHandler(connection); // handle the connection
             connection.close(); //close the connection after handling it
         }
     }catch(Exception e) {
         msg("Error accepting connection !!!");
     }   
 }

/**
 * Prints out messages on the server console window
 *
 */
 private void msg(String str){
     System.out.println(str);
 }

/**
 * Main method that starts the execution of the server. It takes in 
 * command line arguments for the port number of the server.
 */
 public static void main(String args[]) {

     int port = 0;

     if(args.length < 1) { // if no arguments are given then use default
         System.out.println("Using default port = 4321");
         port = 4321;
     }else{
         try{ // else try to get an integer port number from command line
             port = Integer.parseInt(args[0]);
         }catch (Exception e) {
             System.out.println(" Usage : java Server port_number ");
             System.exit(1);
         }
     }
     new Server(port); // create a new Server object on the port number
 }

}// end of class
